using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Sisulizer; // Contains conversion functions

namespace Converter
{
  /// <summary>
  /// Interaction logic for Window1.xaml
  /// </summary>

  public partial class Window1 : System.Windows.Window
  {
    private bool updating = false;

    public Window1()
    {
      InitializeComponent();
    }

    public TQuantity Quantity
    {
      get
      {
        return (TQuantity)typeCombo.SelectedIndex;
      }
    }

    public TLaMetricLength MetricLength
    {
      get
      {
        return (TLaMetricLength)metricCombo.SelectedIndex;
      }
    }

    public TLaUsLength UsLength
    {
      get
      {
        return (TLaUsLength)usCombo.SelectedIndex;
      }
    }

    public TLaMetricTemperature MetricTemperature
    {
      get
      {
        return (TLaMetricTemperature)metricCombo.SelectedIndex;
      }
    }

    public TLaUsTemperature UsTemperature
    {
      get
      {
        return (TLaUsTemperature)usCombo.SelectedIndex;
      }
    }

    public double MetricValue
    {
      get
      {
        try
        {
          return Convert.ToDouble(metricText.Text);
        }
        catch
        {
          return 0;
        }
      }

      set
      {
        try
        {
          updating = true;
          metricText.Text = value.ToString("F");
        }
        finally
        {
          updating = false;
        }
      }
    }

    public double UsValue
    {
      get
      {
        try
        {
          return Convert.ToDouble(usText.Text);
        }
        catch
        {
          return 0;
        }
      }

      set
      {
        try
        {
          updating = true;
          usText.Text = value.ToString("F");
        }
        finally
        {
          updating = false;
        }
      }
    }

    private void addMetric(string value)
    {
      metricCombo.Items.Add(value);
    }

    private void addUs(string value)
    {
      usCombo.Items.Add(value);
    }

    private void WindowLoaded(object sender, RoutedEventArgs e)
    {
      Uri iconUri = new Uri("pack://application:,,,/Converter.ico", UriKind.RelativeOrAbsolute);
      Icon = BitmapFrame.Create(iconUri);

      typeCombo.Items.Add("Length");
      typeCombo.Items.Add("Temperature");
      typeCombo.SelectedIndex = 0;
    }

    private void ExitClicked(object sender, RoutedEventArgs e)
    {
      App.Current.Shutdown();
    }

    private void AboutClicked(object sender, RoutedEventArgs e)
    {
      MessageBox.Show(
        "Converter is an application that converts metric units to US units and back.",
        "About Converter",
        MessageBoxButton.OK,
        MessageBoxImage.Information);
    }

    public void TypeChanged(object sender, SelectionChangedEventArgs e)
    {
      metricCombo.Items.Clear();
      usCombo.Items.Clear();

      switch (Quantity)
      {
        case TQuantity.qaLength:
          addMetric("Kilometers");
          addMetric("Meters");
          addMetric("Centimeters");
          addMetric("Millimeters");
          addUs("Miles");
          addUs("Yards");
          addUs("Feet");
          addUs("Inches");
          break;

        case TQuantity.qaTemperature:
          addMetric("Celsius");
          addMetric("Kelvin");
          addUs("Fahrenheit");
          break;
      }

      metricCombo.SelectedIndex = 0;
      usCombo.SelectedIndex = 0;
    }

    private void MetricTextChanged(object sender, TextChangedEventArgs e)
    {
      if (updating || (metricText.Text == ""))
        return;

      switch (Quantity)
      {
        case TQuantity.qaLength:
          UsValue = Measurements.metricLengthToUsLength(MetricValue, MetricLength, UsLength);
          break;

        case TQuantity.qaTemperature:
          UsValue = Measurements.metricTemperatureToUsTemperature(MetricValue, MetricTemperature, UsTemperature);
          break;
      }
    }

    private void UsTextChanged(object sender, TextChangedEventArgs e)
    {
      if (updating || (usText.Text == ""))
        return;

      switch (Quantity)
      {
        case TQuantity.qaLength:
          MetricValue = Measurements.usLengthToMetricLength(UsValue, UsLength, MetricLength);
          break;

        case TQuantity.qaTemperature:
          MetricValue = Measurements.usTemperatureToMetricTemperature(UsValue, UsTemperature, MetricTemperature);
          break;
      }
    }
  }
}