/*
 * This control contains three properties that contain structured data
 * 
 * 1) Expression
 *    This contains three strings separated by ; characters. 
 *    User enters regular expression that describe the data format. When scanning Sisulizer
 *    uses the expression to break the data into pieces that are localized.
 * 
 * 2) Properties
 *    This contains XML data.
 * 
 * 3) HtmlData
 *    This contains HTML data.
 * 
 * All properties are hidden in the Properties editor but their values are stored in the
 * .resx file
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace Control
{
  public class MyControl: GroupBox
  {
    private String expression = "One;Two;Three";
    private String properties = "<?xml version=\"1.0\"?><data caption=\"Name\"><header>Header</header><footer>Footer</footer></data>";
    private String htmlData = "<html><head><title>Countries</title></head><body><h1>Countries</h1><p>The United States of America is a federal republic of 50 states, located primary on central North America.</p></body></html>";

    protected override void OnPaint(PaintEventArgs e)
    {
      base.OnPaint(e);

      int i;
      string part;
      string str = expression;

      i = str.IndexOf(';');
      part = str.Substring(0, i);
      e.Graphics.DrawString(part, Font, new SolidBrush(Color.Black), 8, 16);

      str = str.Remove(0, i + 1);
      i = str.IndexOf(';');
      part = str.Substring(0, i);
      e.Graphics.DrawString(part, Font, new SolidBrush(Color.Black), 8, 32);

      str = str.Remove(0, i + 1);
      e.Graphics.DrawString(str, Font, new SolidBrush(Color.Black), 8, 48);
    }

    [Browsable(false), Localizable(true)]
    public String Expression
    {
      get
      {
        return expression;
      }

      set
      {
        expression = value;
      }
    }

    [Browsable(false), Localizable(true)]
    public String Properties
    {
      get
      {
        return properties;
      }

      set
      {
        properties = value;
      }
    }

    [Browsable(false), Localizable(true)]
    public String HtmlData
    {
      get
      {
        return htmlData;
      }

      set
      {
        htmlData = value;
      }
    }
  }
}
