using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Resources;
using Sisulizer;

namespace Converter
{
  public partial class Form1 : Form
  {
    private bool updating = false;

    public Form1()
    {
      InitializeComponent();
    }
  
    public TQuantity Quantity
    {
      get
      {
        return (TQuantity)typeCombo.SelectedIndex;
      }
    }

    public TLaMetricLength MetricLength
    {
      get
      {
        return (TLaMetricLength)metricCombo.SelectedIndex;
      }
    }

    public TLaUsLength UsLength
    {
      get
      {
        return (TLaUsLength)usCombo.SelectedIndex;
      }
    }

    public TLaMetricTemperature MetricTemperature
    {
      get
      {
        return (TLaMetricTemperature)metricCombo.SelectedIndex;
      }
    }

    public TLaUsTemperature UsTemperature
    {
      get
      {
        return (TLaUsTemperature)usCombo.SelectedIndex;
      }
    }

    public double MetricValue
    {
      get
      {
        try
        {
          return Convert.ToDouble(metricText.Text);
        }
        catch
        {
          return 0;
        }
      }

      set
      {
        try
        {
          updating = true;
          metricText.Text = value.ToString("F");
        }
        finally
        {
          updating = false;
        }
      }
    }

    public double UsValue
    {
      get
      {
        try
        {
          return Convert.ToDouble(usText.Text);
        }
        catch
        {
          return 0;
        }
      }

      set
      {
        try
        {
          updating = true;
          usText.Text = value.ToString("F");
        }
        finally
        {
          updating = false;
        }
      }
    }

    private void Form1_Load(object sender, EventArgs e)
    {
      typeCombo.Items.Clear();
      typeCombo.Items.Add(Properties.Resources.Length);
      typeCombo.Items.Add(Properties.Resources.Temperature);
      typeCombo.SelectedIndex = 0;
    }

    private void addMetric(string value)
    {
      metricCombo.Items.Add(value);
    }

    private void addUs(string value)
    {
      usCombo.Items.Add(value);
    }

    private void typeCombo_SelectedIndexChanged(object sender, EventArgs e)
    {
      metricCombo.Items.Clear();
      usCombo.Items.Clear();

      ResourceManager rm = new ResourceManager("Converter.Properties.Resources", this.GetType().Assembly);
      string str = rm.GetString("Centimeters");

      switch (Quantity)
      {
        case TQuantity.qaLength:
          addMetric(Properties.Resources.Kilometers);
          addMetric(Properties.Resources.Meters);
          addMetric(Properties.Resources.Centimeters);
          addMetric(Properties.Resources.Millimeters);
          addUs(Properties.Resources.Miles);
          addUs(Properties.Resources.Yards);
          addUs(Properties.Resources.Feet);
          addUs(Properties.Resources.Inches);
          break;

        case TQuantity.qaTemperature:
          addMetric(Properties.Resources.Celsius);
          addMetric(Properties.Resources.Kelvin);
          addUs(Properties.Resources.Fahrenheit);
          break;
      }

      metricCombo.SelectedIndex = 0;
      usCombo.SelectedIndex = 0;
    }

    private void metricText_TextChanged(object sender, EventArgs e)
    {
      if (updating || (metricText.Text == ""))
        return;

      switch (Quantity)
      {
        case TQuantity.qaLength:
          UsValue = Measurements.metricLengthToUsLength(MetricValue, MetricLength, UsLength);
          break;

        case TQuantity.qaTemperature:
          UsValue = Measurements.metricTemperatureToUsTemperature(MetricValue, MetricTemperature, UsTemperature);
          break;
      }
    }

    private void usText_TextChanged(object sender, EventArgs e)
    {
      if (updating || (usText.Text == ""))
        return;

      switch (Quantity)
      {
        case TQuantity.qaLength:
          MetricValue = Measurements.usLengthToMetricLength(UsValue, UsLength, MetricLength);
          break;

        case TQuantity.qaTemperature:
          MetricValue = Measurements.usTemperatureToMetricTemperature(UsValue, UsTemperature, MetricTemperature);
          break;
      }
    }

    private void exitToolStripMenuItem_Click(object sender, EventArgs e)
    {
      Application.Exit();
    }

    private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
    {
      MessageBox.Show(
        Properties.Resources.About_Message,
        Properties.Resources.About_Converter,
        MessageBoxButtons.OK,
        MessageBoxIcon.Information);
    }

    private void languageToolStripMenuItem_Click(object sender, EventArgs e)
    {
    }
  }
}