/*
  Copyrights 2005-2011 Sisulizer
*/

using System.Globalization;
using System.Threading;
using System.Windows.Forms;

namespace Sisulizer.WindowsForms
{
  public partial class SelectLanguage : Form
  {
    private CultureInfo[] cultureInfos = null;

    public SelectLanguage(string originalLanguage)
    {
      InitializeComponent();
      Populate(originalLanguage);
    }

    public CultureInfo Language
    {
      get
      {
        return cultureInfos[listBox.SelectedIndex];
      }
      set
      {
        for (int i = 0; i < cultureInfos.Length; i++)
        {
          if (cultureInfos[i] == value)
          {
            listBox.SelectedIndex = i;
            return;
          }
        }

        for (int i = 0; i < cultureInfos.Length; i++)
        {
          if (cultureInfos[i].TwoLetterISOLanguageName == value.TwoLetterISOLanguageName)
          {
            listBox.SelectedIndex = i;
            return;
          }
        }

        if (listBox.Items.Count > 0)
          listBox.SelectedIndex = 0;
      }
    }

    private void AddLanguage(CultureInfo cultureInfo)
    {
      cultureInfos[listBox.Items.Count] = cultureInfo;
      listBox.Items.Add(cultureInfo.NativeName);
    }

    public void Populate(string originalLanguage)
    {
      CultureInfo[] languages = Sisulizer.Resources.GetAvailableLanguages();

      cultureInfos = new CultureInfo[languages.Length + 1];
      listBox.BeginUpdate();
      AddLanguage(new CultureInfo(originalLanguage));

      for (int i = 0; i < languages.Length; i++)
        AddLanguage(languages[i]);

      listBox.EndUpdate();

      Language = Thread.CurrentThread.CurrentUICulture;
    }

    static public bool Select(string originalLanguage)
    {
      SelectLanguage dialog = new SelectLanguage(originalLanguage);

      if (dialog.ShowDialog() == DialogResult.OK)
      {
        Translator.SetUILanguage(dialog.Language);
        return true;
      }
      else
      {
        return false;
      }
    }
  }
}