#include <QtGui>
#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent, Qt::WFlags flags): QMainWindow(parent, flags)
{
  ui.setupUi(this);

  updating = false;

  // Set slots
  connect(ui.actionExit, SIGNAL(triggered()), this, SLOT(close()));
  connect(ui.actionAbout, SIGNAL(triggered()), this, SLOT(about()));
  connect(ui.typeCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(typeChanged(int)));
  connect(ui.metricUnitCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(metricUnitChanged(int)));
  connect(ui.metricAmountEdit, SIGNAL(textChanged()), this, SLOT(metricAmountChanged()));
  connect(ui.usUnitCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(usUnitChanged(int)));
  connect(ui.usAmountEdit, SIGNAL(textChanged()), this, SLOT(usAmountChanged()));

  // Populate type combo
  ui.typeCombo->addItem(tr("Length"), Length);
  ui.typeCombo->addItem(tr("Temperature"), Temperature);
}

MainWindow::~MainWindow()
{
}

Quantity MainWindow::getQuantity()
{
  return (Quantity)ui.typeCombo->itemData(ui.typeCombo->currentIndex()).toInt();
}

MetricLength MainWindow::getMetricLength()
{
  return (MetricLength)ui.metricUnitCombo->itemData(ui.metricUnitCombo->currentIndex()).toInt();
}

UsLength MainWindow::getUsLength()
{
  return (UsLength)ui.usUnitCombo->itemData(ui.usUnitCombo->currentIndex()).toInt();
}

MetricTemperature MainWindow::getMetricTemperature()
{
  return (MetricTemperature)ui.metricUnitCombo->itemData(ui.metricUnitCombo->currentIndex()).toInt();
}

UsTemperature MainWindow::getUsTemperature()
{
  return (UsTemperature)ui.usUnitCombo->itemData(ui.usUnitCombo->currentIndex()).toInt();
}

double MainWindow::getMetricValue()
{
  QString value = ui.metricAmountEdit->toPlainText();
  return value.toDouble();
}

void MainWindow::setMetricValue(double value)
{
  QString str;
  str.setNum(value, 'f', 2);

  updating = true;
  ui.metricAmountEdit->setText(str);
  updating = false;
}

double MainWindow::getUsValue()
{
  QString value = ui.usAmountEdit->toPlainText();
  return value.toDouble();
}

void MainWindow::setUsValue(double value)
{
  QString str;
  str.setNum(value, 'f', 2);

  updating = true;
  ui.usAmountEdit->setText(str);
  updating = false;
}

QString MainWindow::getMetricLengthStr(MetricLength value)
{
  switch (value)
  {
    case Km:
      return tr("Kilometers");

    case m:
      return tr("Meters");

    case cm:
      return tr("Centimeters");

    case mm:
      return tr("Millimeters");

    default:
      return "";
  }
}

QString MainWindow::getUsLengthStr(UsLength value)
{
  switch (value)
  {
    case Mile:
      return tr("Miles");

    case Yard:
      return tr("Yards");

    case Foot:
      return tr("Feet");

    case Inch:
      return tr("Inches");

    default:
      return "";
  }
}

QString MainWindow::getMetricTemperatureStr(MetricTemperature value)
{
  switch (value)
  {
    case Celsius:
      return tr("Celsius");

    case Kelvin:
      return tr("Kelvin");

    default:
      return "";
  }
}

QString MainWindow::getUsTemperatureStr(UsTemperature value)
{
  return tr("Fahrenheit");
}

void MainWindow::typeChanged(int index)
{
  ui.metricUnitCombo->clear();
  ui.usUnitCombo->clear();

  switch (getQuantity())
  {
    case Length:
    {
      for (int i = Km; i <= mm; i++)
      {
        ui.metricUnitCombo->addItem(getMetricLengthStr((MetricLength)i), i);
      }

      for (int i = Mile; i <= Inch; i++)
      {
        ui.usUnitCombo->addItem(getUsLengthStr((UsLength)i), i);
      }

      break;
    }

    case Temperature:
    {
      for (int i = Celsius; i <= Kelvin; i++)
      {
        ui.metricUnitCombo->addItem(getMetricTemperatureStr((MetricTemperature)i), i);
      }

      for (int i = Fahrenheit; i <= Fahrenheit; i++)
      {
        ui.usUnitCombo->addItem(getUsTemperatureStr((UsTemperature)i), i);
      }

      break;
    }
  }

  ui.metricUnitCombo->setCurrentIndex(0);
  ui.usUnitCombo->setCurrentIndex(0);
}

void MainWindow::metricUnitChanged(int index)
{
  metricAmountChanged();
}

void MainWindow::metricAmountChanged()
{
  if (updating || (ui.metricAmountEdit->toPlainText().length() == 0))
    return;

  switch (getQuantity())
  {
    case Length:
      setUsValue(metricLengthToUsLength(getMetricValue(), getMetricLength(), getUsLength()));
      break;

    case Temperature:
      setUsValue(metricTemperatureToUsTemperature(getMetricValue(), getMetricTemperature(), getUsTemperature()));
      break;
  }
}

void MainWindow::usUnitChanged(int index)
{
  usAmountChanged();
}

void MainWindow::usAmountChanged()
{
  if (updating || (ui.usAmountEdit->toPlainText().length() == 0))
    return;

  switch (getQuantity())
  {
    case Length:
      setMetricValue(usLengthToMetricLength(getUsValue(), getUsLength(), getMetricLength()));
      break;

    case Temperature:
      setMetricValue(usTemperatureToMetricTemperature(getUsValue(), getUsTemperature(), getMetricTemperature()));
      break;
  }
}

void MainWindow::about()
{
  QMessageBox::about(this, tr("About Converter"), tr("Converter is an application that converts metric units to US units and back."));
}
