#include <QtGui>
#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent, Qt::WFlags flags): QMainWindow(parent, flags)
{
  ui.setupUi(this);

  updating = false;

  // Set slots
  connect(ui.actionExit, SIGNAL(triggered()), this, SLOT(close()));
  connect(ui.actionAbout, SIGNAL(triggered()), this, SLOT(about()));
  connect(ui.typeCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(typeChanged(int)));
  connect(ui.metricUnitCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(metricUnitChanged(int)));
  connect(ui.metricAmountEdit, SIGNAL(textChanged()), this, SLOT(metricAmountChanged()));
  connect(ui.usUnitCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(usUnitChanged(int)));
  connect(ui.usAmountEdit, SIGNAL(textChanged()), this, SLOT(usAmountChanged()));

  // Populate type combo
  ui.typeCombo->addItem("Length", Length);
  ui.typeCombo->addItem("Temperature", Temperature);
}

MainWindow::~MainWindow()
{
}

Quantity MainWindow::getQuantity()
{
  return (Quantity)ui.typeCombo->itemData(ui.typeCombo->currentIndex()).toInt();
}

MetricLength MainWindow::getMetricLength()
{
  return (MetricLength)ui.metricUnitCombo->itemData(ui.metricUnitCombo->currentIndex()).toInt();
}

UsLength MainWindow::getUsLength()
{
  return (UsLength)ui.usUnitCombo->itemData(ui.usUnitCombo->currentIndex()).toInt();
}

MetricTemperature MainWindow::getMetricTemperature()
{
  return (MetricTemperature)ui.metricUnitCombo->itemData(ui.metricUnitCombo->currentIndex()).toInt();
}

UsTemperature MainWindow::getUsTemperature()
{
  return (UsTemperature)ui.usUnitCombo->itemData(ui.usUnitCombo->currentIndex()).toInt();
}

double MainWindow::getMetricValue()
{
  QString value = ui.metricAmountEdit->toPlainText();
  return value.toDouble();
}

void MainWindow::setMetricValue(double value)
{
  QString str;
  str.setNum(value, 'f', 2);

  updating = true;
  ui.metricAmountEdit->setText(str);
  updating = false;
}

double MainWindow::getUsValue()
{
  QString value = ui.usAmountEdit->toPlainText();
  return value.toDouble();
}

void MainWindow::setUsValue(double value)
{
  QString str;
  str.setNum(value, 'f', 2);

  updating = true;
  ui.usAmountEdit->setText(str);
  updating = false;
}

static const char* METRIC_LENGHTS[] = { "Kilometers", "Meters", "Centimeters", "Millimeters"};
static const char* US_LENGHTS[] = { "Miles", "Yards", "Feet", "Inches"};
static const char* METRIC_TEMPERATURES[] = { "Celsius", "Kelvin"};
static const char* US_TEMPERATURES[] = { "Fahrenheit"};

void MainWindow::typeChanged(int index)
{
  ui.metricUnitCombo->clear();
  ui.usUnitCombo->clear();

  switch (getQuantity())
  {
    case Length:
    {
      for (int i = Km; i <= mm; i++)
      {
        ui.metricUnitCombo->addItem(METRIC_LENGHTS[i], i);
      }

      for (int i = Mile; i <= Inch; i++)
      {
        ui.usUnitCombo->addItem(US_LENGHTS[i], i);
      }

      break;
    }

    case Temperature:
    {
      for (int i = Celsius; i <= Kelvin; i++)
      {
        ui.metricUnitCombo->addItem(METRIC_TEMPERATURES[i], i);
      }

      for (int i = Fahrenheit; i <= Fahrenheit; i++)
      {
        ui.usUnitCombo->addItem(US_TEMPERATURES[i], i);
      }

      break;
    }
  }

  ui.metricUnitCombo->setCurrentIndex(0);
  ui.usUnitCombo->setCurrentIndex(0);
}

void MainWindow::metricUnitChanged(int index)
{
  metricAmountChanged();
}

void MainWindow::metricAmountChanged()
{
  if (updating || (ui.metricAmountEdit->toPlainText().length() == 0))
    return;

  switch (getQuantity())
  {
    case Length:
      setUsValue(metricLengthToUsLength(getMetricValue(), getMetricLength(), getUsLength()));
      break;

    case Temperature:
      setUsValue(metricTemperatureToUsTemperature(getMetricValue(), getMetricTemperature(), getUsTemperature()));
      break;
  }
}

void MainWindow::usUnitChanged(int index)
{
  usAmountChanged();
}

void MainWindow::usAmountChanged()
{
  if (updating || (ui.usAmountEdit->toPlainText().length() == 0))
    return;

  switch (getQuantity())
  {
    case Length:
      setMetricValue(usLengthToMetricLength(getUsValue(), getUsLength(), getMetricLength()));
      break;

    case Temperature:
      setMetricValue(usTemperatureToMetricTemperature(getUsValue(), getUsTemperature(), getMetricTemperature()));
      break;
  }
}

void MainWindow::about()
{
  QMessageBox::about(this, "About Converter", "Converter is an application that converts metric units to US units and back.");
}
