Attribute VB_Name = "LaCommon"
Public Const MILE_AS_METERS = 1609.3
Public Const YARD_AS_METERS = 0.9144

Public Const INCH_IN_FEET = 12
Public Const FEET_IN_YARD = 3
Public Const YARDS_IN_MILE = 1760

Public Const CELSIUS_OFFSET = 273.15
Public Const FAHRENHEIT_OFFSET = 32
Public Const FAHRENHEIT_FACTOR = 1.8

Enum TQuantity
  qaLength = 0
  qaTemperature = 1
End Enum

Enum TLaMetricLength
  mlKm = 0
  mlM = 1
  mlCm = 2
  mlMm = 3
End Enum

Enum TLaUsLength
  ulMile = 0
  ulYard = 1
  ulFoot = 2
  ulInch = 3
End Enum

Enum TLaMetricTemperature
  mtCelsius = 0
  mtKelvin = 1
End Enum

Enum TLaUsTemperature
  utFahrenheit = 0
End Enum

Public Function MetricLengthToUsLength(value As Double, metricLength As TLaMetricLength, usLength As TLaUsLength) As Double
  Dim meters As Double
  Dim yards As Double
  
  Select Case metricLength
    Case mlKm
      meters = 1000 * value
      
    Case mlCm
      meters = value / 100
     
    Case mlMm
      meters = value / 1000
    Case Else
      meters = value
  End Select

  yards = meters / YARD_AS_METERS

  Select Case usLength
    Case ulMile
      MetricLengthToUsLength = yards / YARDS_IN_MILE
      
    Case ulFoot
      MetricLengthToUsLength = FEET_IN_YARD * yards
      
    Case ulInch
      MetricLengthToUsLength = FEET_IN_YARD * INCH_IN_FEET * yards
      
    Case Else
      MetricLengthToUsLength = yards
  End Select
End Function

Public Function UsLengthToMetricLength(value As Double, usLength As TLaUsLength, metricLength As TLaMetricLength) As Double
  Dim meters As Double
  Dim yards As Double

  Select Case usLength
    Case ulMile
      yards = YARDS_IN_MILE * value
      
    Case ulFoot
      yards = value / FEET_IN_YARD
      
    Case ulInch
      yards = value / (FEET_IN_YARD * INCH_IN_FEET)
      
    Case Else
      yards = value
  End Select

  meters = YARD_AS_METERS * yards

  Select Case metricLength
    Case mlKm
      Result = meters / 1000
      
    Case mlCm
      Result = 100 * meters
      
    Case mlMm
      Result = 1000 * meters
  
    Case Else
      Result = meters
  End Select
End Function

Public Function MetricTemperatureToUsTemperature(value As Double, metricTemperature As TLaMetricTemperature, usTemperature As TLaUsTemperature) As Double
  Dim celsius As Double
  Dim fahrenheit As Double
  
  Select Case metricTemperature
    Case mtKelvin
      celsius = value - CELSIUS_OFFSET
    Case Else
      celsius = value
  End Select

  fahrenheit = FAHRENHEIT_FACTOR * celsius + FAHRENHEIT_OFFSET

  MetricTemperatureToUsTemperature = fahrenheit
End Function

Public Function UsTemperatureToMetricTemperature(value As Double, usTemperature As TLaUsTemperature, metricTemperature As TLaMetricTemperature) As Double
  Dim celsius As Double
  Dim fahrenheit As Double
  
  fahrenheit = value

  celsius = (fahrenheit - FAHRENHEIT_OFFSET) / FAHRENHEIT_FACTOR

  Select Case metricTemperature
    Case mtKelvin
      UsTemperatureToMetricTemperature = celsius + CELSIUS_OFFSET
    Case Else
      UsTemperatureToMetricTemperature = celsius
  End Select
End Function
