unit Main;

{$WARN UNSAFE_CAST OFF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Menus, ElComponent, ElCaption, ElCLabel, ElLabel,
  ElEdits, ElBtnEdit, ElCombos, ElXPThemedControl, ElStatBar, LaCommon, ElMenus;

type
  TMainForm = class(TForm)
    MainMenu: TElMainMenu;
    FileMenu: TElMenuItem;
    LanguageMenu: TElMenuItem;
    SetDefaultLanguageMenu: TElMenuItem;
    N1: TElMenuItem;
    ExitMenu: TElMenuItem;
    HelpMenu: TElMenuItem;
    AboutMenu: TElMenuItem;
    TypeLabel: TElLabel;
    TypeCombo: TElComboBox;
    MetricUnitLabel: TElLabel;
    MetricCombo: TElComboBox;
    MetricAmountLabel: TElLabel;
    MetricEdit: TElEdit;
    UsUnitLabel: TElLabel;
    UsCombo: TElComboBox;
    UsAmountLabel: TElLabel;
    UsEdit: TElEdit;
    StatusBar: TElStatusBar;
    ElFormCaption: TElFormCaption;
    procedure FormCreate(Sender: TObject);
    procedure TypeComboChange(Sender: TObject);
    procedure MetricEditChange(Sender: TObject);
    procedure UsEditChange(Sender: TObject);
    procedure LanguageMenuClick(Sender: TObject);
    procedure SetDefaultLanguageMenuClick(Sender: TObject);
    procedure ExitMenuClick(Sender: TObject);
    procedure AboutMenuClick(Sender: TObject);

  private
    FUpdating: Boolean;
    
    function GetQuantity: TQuantity;

    function GetMetricLength: TLaMetricLength;
    function GetUsLength: TLaUsLength;

    function GetMetricTemperature: TLaMetricTemperature;
    function GetUsTemperature: TLaUsTemperature;

    function GetMetricValue: Double;
    procedure SetMetricValue(value: Double);

    function GetUsValue: Double;
    procedure SetUsValue(value: Double);

    procedure ShowHint(Sender: TObject);
    procedure Initialize;

  public
    property Quantity: TQuantity read GetQuantity;
    property MetricLength: TLaMetricLength read GetMetricLength;
    property UsLength: TLaUsLength read GetUsLength;
    property MetricTemperature: TLaMetricTemperature read GetMetricTemperature;
    property UsTemperature: TLaUsTemperature read GetUsTemperature;
    property MetricValue: Double read GetMetricValue write SetMetricValue;
    property UsValue: Double read GetUsValue write SetUsValue;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  LaDialog,
  LaResource;

procedure TMainForm.Initialize;

  function GetQuantity(value: TQuantity): WideString;
  resourcestring
    SLength = 'Length';
    STemperature = 'Temperature';
  begin
    case value of
      qaLength: Result := SLength;
      qaTemperature: Result := STemperature;
    end;
  end;

var
  q: TQuantity;
begin
  TypeCombo.Items.Clear;

  for q := Low(q) to High(q) do
    TypeCombo.Items.AddObject(GetQuantity(q), TObject(q));

  TypeCombo.ItemIndex := 0;
  TypeComboChange(Self);
end;

function TMainForm.GetQuantity: TQuantity;
begin
  Result := TQuantity(TypeCombo.Items.Objects[TypeCombo.ItemIndex]);
end;

function TMainForm.GetMetricLength: TLaMetricLength;
begin
  Result := TLaMetricLength(MetricCombo.Items.Objects[MetricCombo.ItemIndex]);
end;

function TMainForm.GetUsLength: TLaUsLength;
begin
  Result := TLaUsLength(UsCombo.Items.Objects[UsCombo.ItemIndex]);
end;

function TMainForm.GetMetricTemperature: TLaMetricTemperature;
begin
  Result := TLaMetricTemperature(MetricCombo.Items.Objects[MetricCombo.ItemIndex]);
end;

function TMainForm.GetUsTemperature: TLaUsTemperature;
begin
  Result := TLaUsTemperature(UsCombo.Items.Objects[UsCombo.ItemIndex]);
end;

function TMainForm.GetMetricValue: Double;
begin
  try
    Result := StrToFloat(MetricEdit.Text);
  except
    Result := 0;
  end;
end;

procedure TMainForm.SetMetricValue(value: Double);
begin
  FUpdating := True;
  try
    MetricEdit.Text := FloatToStrF(value, ffNumber, 10, 2);
  finally
    FUpdating := False;
  end;
end;

function TMainForm.GetUsValue: Double;
begin
  try
    Result := StrToFloat(UsEdit.Text);
  except
    Result := 0;
  end;
end;

procedure TMainForm.SetUsValue(value: Double);
begin
  FUpdating := True;
  try
    UsEdit.Text := FloatToStrF(value, ffNumber, 10, 2);
  finally
    FUpdating := False;
  end;
end;

procedure TMainForm.ShowHint(Sender: TObject);

  function WideGetLongHint(const hint: WideString): WideString;
  var
    i: Integer;
  begin
    i := Pos('|', hint);

    if i = 0 then
      Result := hint
    else
      Result := Copy(hint, i + 1, Maxint);
  end;

begin
  StatusBar.SimpleText := WideGetLongHint(Application.Hint);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
  Initialize;
end;

procedure TMainForm.TypeComboChange(Sender: TObject);

  procedure AddMetric(const str: WideString; value: Integer);
  begin
    MetricCombo.Items.AddObject(str, TObject(value));
  end;

  procedure AddUs(const str: WideString; value: Integer);
  begin
    UsCombo.Items.AddObject(str, TObject(value));
  end;

  function GetMetricLength(value: TLaMetricLength): WideString;
  resourcestring
    SKilometers = 'Kilometers';
    SMeters = 'Meters';
    SCentimeters = 'Centimeters';
    SMillimeters = 'Millimeters';
  begin
    case value of
      mlKm: Result := SKilometers;
      mlCm: Result := SCentimeters;
      mlMm: Result := SMillimeters;
    else
      Result := SMeters;
    end;
  end;

  function GetUsLength(value: TLaUsLength): WideString;
  resourcestring
    SMiles = 'Miles';
    SYards = 'Yards';
    SFeet = 'Feet';
    SInches = 'Inches';
  begin
    case value of
      ulMile: Result := SMiles;
      ulFoot: Result := SFeet;
      ulInch: Result := SInches;
    else
      Result := SYards;
    end;
  end;

  function GetMetricTemperature(value: TLaMetricTemperature): WideString;
  resourcestring
    SCelsius = 'Celsius';
    SKelvin = 'Kelvin';
  begin
    case value of
      mtKelvin: Result := SKelvin;
    else
      Result := SCelsius;
    end;
  end;

  function GetUsTemperature(value: TLaUsTemperature): WideString;
  resourcestring
    SFahrenheit = 'Fahrenheit';
  begin
    Result := SFahrenheit;
  end;

var
  ml: TLaMetricLength;
  ul: TLaUsLength;
  mt: TLaMetricTemperature;
  ut: TLaUsTemperature;
begin
  MetricCombo.Items.Clear;
  UsCombo.Items.Clear;

  case Quantity of
    qaLength:
    begin
      for ml := Low(ml) to High(ml) do
        AddMetric(GetMetricLength(ml), Ord(ml));

      for ul := Low(ul) to High(ul) do
        AddUs(GetUsLength(ul), Ord(ul));
    end;

    qaTemperature:
    begin
      for mt := Low(mt) to High(mt) do
        AddMetric(GetMetricTemperature(mt), Ord(mt));

      for ut := Low(ut) to High(ut) do
        AddUs(GetUsTemperature(ut), Ord(ut));
    end;
  end;

  MetricCombo.ItemIndex := 0;
  UsCombo.ItemIndex := 0;
end;

procedure TMainForm.MetricEditChange(Sender: TObject);
begin
  if FUpdating or (MetricEdit.Text = '') then
    Exit;

  case Quantity of
    qaLength:
      UsValue := MetricLengthToUsLength(MetricValue, MetricLength, UsLength);

    qaTemperature:
      UsValue := MetricTemperatureToUsTemperature(MetricValue, MetricTemperature, UsTemperature);
  end;
end;

procedure TMainForm.UsEditChange(Sender: TObject);
begin
  if FUpdating or (UsEdit.Text = '') then
    Exit;

  case Quantity of
    qaLength:
      MetricValue := UsLengthToMetricLength(UsValue, UsLength, MetricLength);

    qaTemperature:
      MetricValue := UsTemperatureToMetricTemperature(UsValue, UsTemperature, MetricTemperature);
  end;
end;

procedure TMainForm.ExitMenuClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.AboutMenuClick(Sender: TObject);
resourcestring
  SAboutConverter = 'About Converter';
  SAbout = 'Converter is an application that converts metric units to US units and back.';
begin
  ShowMessage(SAboutConverter + #13#10#13#10 + SAbout);
end;

procedure TMainForm.LanguageMenuClick(Sender: TObject);
begin
  // Check the resource DLL count
  CheckResourceCount;

  // Shows a select language dialog that lets the user to select a new language,
  // and finally load the new resource DLL
  if SelectResourceLocale('EN') then
  begin
    // Save the selected language to the system registry
    SetCurrentDefaultLocaleReg;

    // Initialize the main form again because it was reloaded
    Initialize;
  end;
end;

procedure TMainForm.SetDefaultLanguageMenuClick(Sender: TObject);
begin
  if ClearCurrentDefaultLocaleReg then
  begin
    // Load a new resource DLL and initialize the main form
    SetNewResourceFile;
    Initialize;
  end;
end;

end.
