{
  Copyrights 2005-2007 Sisulizer Oy

  This sample application shows how to create a multilingual ReportBuilder
  application. The application accesses the report in two ways.

  1) Embedded report data

  The report data is embedded to the form inside the TppReport component.

  2) External report file

  The report file (.rtm) is loaded and then shown.
  There is one report file for each locale.

  Country.rtm    = Original report file
  de\Country.rtm = German report file (*)
  fi\Country.rtm = Finnish report file (*)
  ja\Country.rtm = Japanese report file (*)

  (*) Sisulizer creates these
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DB, DBTables, ppVar, ppBands, ppCtrls, ppPrnabl, ppClass,
  ppCache, ppProd, ppReport, ppComm, ppRelatv, ppDB, ppDBPipe;

type
  TForm1 = class(TForm)
    CountryQuery: TQuery;
    CountryDataSource: TDataSource;
    CountryPipeline: TppDBPipeline;
    CountryReport: TppReport;
    PreviewEmbeddedButton: TButton;
    PreviewExternalButton: TButton;
    LanguageButton: TButton;
    ppHeaderBand1: TppHeaderBand;
    ppDetailBand1: TppDetailBand;
    ppFooterBand1: TppFooterBand;
    ppTitleBand1: TppTitleBand;
    ppLabel1: TppLabel;
    ppSystemVariable2: TppSystemVariable;
    ppDBText1: TppDBText;
    ppLabel2: TppLabel;
    ppDBText2: TppDBText;
    ppLabel3: TppLabel;
    ppDBText3: TppDBText;
    ppLabel4: TppLabel;
    ppDBText4: TppDBText;
    ppLabel5: TppLabel;
    procedure FormCreate(Sender: TObject);
    procedure PreviewEmbeddedButtonClick(Sender: TObject);
    procedure PreviewExternalButtonClick(Sender: TObject);
    procedure LanguageButtonClick(Sender: TObject);

  private
    function GetLanguage: String;

    procedure Initialize;

  public
    property Language: String read GetLanguage;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  ppTypes,
  LaDialog,
  LaResource;

function TForm1.GetLanguage: String;
begin
  // Returns the active language code
  Result := LowerCase(GetActiveResourceLocale('EN'));
end;

procedure TForm1.Initialize;
begin
  if CountryQuery.Active then
    CountryQuery.Close;

  // Set query that shows database rows that use the selected language
  CountryQuery.SQL.Text := Format('SELECT * FROM Country WHERE Language=''%s''', [Language]);
  CountryQuery.Open;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Initialize;
end;

procedure TForm1.PreviewEmbeddedButtonClick(Sender: TObject);
begin
  CountryReport.Print;
end;

procedure TForm1.PreviewExternalButtonClick(Sender: TObject);
const
  FILE_NAME_C = 'Country.rtm';
var
  report: TppReport;
begin
  report := TppReport.Create(nil);
  try
    // Load the report file from the language specific directory.
    // If the file does not exist use the original report file.
    report.Template.FileName := Language + '\' + FILE_NAME_C;

    if not FileExists(report.Template.FileName) then
      report.Template.FileName := FILE_NAME_C;

    report.Template.LoadFromFile;
    report.DataPipeline := CountryPipeline;
    report.PreviewFormSettings.ZoomSetting := zsPageWidth;
    report.Print;
  finally
    report.Free;
  end;
end;

procedure TForm1.LanguageButtonClick(Sender: TObject);
begin
  // Perform runtime language change
  CheckResourceCount;

  if SelectResourceLocale('EN') then
    Initialize;
end;

end.
