{
  Copyrights 2005-2007 Sisulizer Ltd.

  A dialog that shows the available resource DLLs and lets the user to select one.
}

{$I LaDef.inc}

unit LaDialog;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, Dialogs, StdCtrls, TntForms,
  TntStdCtrls, LaResource;

type
  TLaLanguageDialog = class(TTntForm)
    List: TTntListBox;
    OkButton: TTntButton;
    CancelButton: TTntButton;
    procedure FormCreate(sender: TObject);
    procedure FormShow(sender: TObject);
    procedure FormDestroy(sender: TObject);
    procedure ListDblClick(sender: TObject);

  private
    FCompatible: Boolean;
    FOriginalLanguage: AnsiString;
    FUseOriginalLanguage: Boolean;

    function GetLanguage(i: Integer): AnsiString;
    function GetLanguageCount: Integer;
    function GetSelectedLangId: Integer;
    function GetSelectedLanguage: AnsiString;

    procedure SetSelectedLanguage(const value: AnsiString);

    procedure AddString(const name: String; const code: AnsiString);
    procedure AddWideString(const name: WideString; const code: AnsiString);

  public
    property LanguageCount: Integer read GetLanguageCount;
    property Languages[i: Integer]: AnsiString read GetLanguage;
    property Compatible: Boolean read FCompatible write FCompatible;
    property OriginalLanguage: AnsiString read FOriginalLanguage write FOriginalLanguage;
    property UseOriginalLanguage: Boolean read FUseOriginalLanguage write FUseOriginalLanguage;
    property SelectedLanguage: AnsiString read GetSelectedLanguage write SetSelectedLanguage;
    property SelectedLangId: Integer read GetSelectedLangId;
  end;

  TLaDialogOption = (doUseCurrentLanguage, doCompatible);
  TLaDialogOptions = set of TLaDialogOption;

function SelectResourceLocale(
  const originalLanguage: String = '';
  dialogOptions: TLaDialogOptions = [];
  resourceOptions: TLaResourceOptions = [];
  position: TPosition = poMainFormCenter): Boolean; overload;

function SelectResourceLocale(
  originalLanguage: Integer = 0;
  dialogOptions: TLaDialogOptions = [];
  resourceOptions: TLaResourceOptions = [];
  position: TPosition = poScreenCenter): Boolean; overload;

function AskResourceLocale(
  var language: String;
  originalLanguage: String = '';
  options: TLaDialogOptions = [];
  position: TPosition = poScreenCenter): Boolean; overload;

function AskResourceLocale(
  var locale: LCID;
  originalLanguage: Integer = 0;
  options: TLaDialogOptions = [];
  position: TPosition = poScreenCenter): Boolean; overload;

implementation

{$R *.dfm}

uses
  LaCommon;

function SelectResourceLocale(
  const originalLanguage: String;
  dialogOptions: TLaDialogOptions;
  resourceOptions: TLaResourceOptions;
  position: TPosition): Boolean;
var
  language: String;
begin
  Result := AskResourceLocale(language, originalLanguage, dialogOptions, position);

  if Result then
  begin
    if language = originalLanguage then
      language := '';

    Result := SetNewResourceFile(language, resourceOptions);
  end;
end;

function SelectResourceLocale(
  originalLanguage: Integer;
  dialogOptions: TLaDialogOptions;
  resourceOptions: TLaResourceOptions;
  position: TPosition): Boolean;
var
  locale: LCID;
begin
  Result := AskResourceLocale(locale, originalLanguage, dialogOptions, position);

  if Result then
  begin
    if Integer(locale) = originalLanguage then
      locale := 0;

    Result := SetNewResourceFile(locale, resourceOptions);
  end;
end;

function AskResourceLocale(
  var language: String;
  originalLanguage: String;
  options: TLaDialogOptions;
  position: TPosition): Boolean;
var
  dialog: TLaLanguageDialog;
begin
  dialog := TLaLanguageDialog.Create(nil);
  try
    dialog.Position := position;
    dialog.UseOriginalLanguage := not (doUseCurrentLanguage in options);
    dialog.Compatible := doCompatible in options;
    dialog.OriginalLanguage := originalLanguage;

    Result := dialog.ShowModal = mrOk;

    if Result then
      language := dialog.SelectedLanguage;
  finally
    dialog.Free;
  end;
end;

function AskResourceLocale(
  var locale: LCID;
  originalLanguage: Integer;
  options: TLaDialogOptions;
  position: TPosition): Boolean;
var
  dialog: TLaLanguageDialog;
begin
  dialog := TLaLanguageDialog.Create(nil);
  try
    dialog.Position := position;
    dialog.UseOriginalLanguage := not (doUseCurrentLanguage in options);
    dialog.Compatible := doCompatible in options;
    dialog.OriginalLanguage := LocaleToExtension(originalLanguage);

    Result := dialog.ShowModal = mrOk;

    if Result then
      locale := dialog.SelectedLangId;
  finally
    dialog.Free;
  end;
end;

function TLaLanguageDialog.GetLanguageCount: Integer;
begin
  Result := List.Items.Count;
end;

function TLaLanguageDialog.GetLanguage(i: Integer): AnsiString;
begin
  Result := TLanguageListItem(List.Items.Objects[i]).Code;
end;

function TLaLanguageDialog.GetSelectedLangId: Integer;
begin
  Result := ExtensionToLocale(SelectedLanguage);
end;

function TLaLanguageDialog.GetSelectedLanguage: AnsiString;
begin
  if List.ItemIndex >= 0 then
    Result := Languages[List.ItemIndex]
  else
    Result := '';
end;

procedure TLaLanguageDialog.SetSelectedLanguage(const value: AnsiString);
var
  i: Integer;
begin
  for i := 0 to LanguageCount - 1 do
    if Languages[i] = value then
    begin
      List.ItemIndex := i;
      Break;
    end;
end;

procedure TLaLanguageDialog.AddString(const name: String; const code: AnsiString);
begin
  List.Items.AddObject(name, TLanguageListItem.Create(code, name));
end;

procedure TLaLanguageDialog.AddWideString(const name: WideString; const code: AnsiString);
begin
  List.Items.AddObject(name, TLanguageListItem.Create(code, name));
end;

procedure TLaLanguageDialog.FormCreate(Sender: TObject);
begin
  FCompatible := False;
  FOriginalLanguage := '';
  FUseOriginalLanguage := False;
end;

procedure TLaLanguageDialog.FormShow(Sender: TObject);

  procedure Process(const code: AnsiString);
  var
    locale: Integer;
  begin
    locale := ExtensionToLocale(code);

    if IsNtOrLater then
      AddWideString(GetWindowsLocaleWideName(locale, FUseOriginalLanguage), code)
    else
      AddString(GetWindowsLocaleName(locale, FUseOriginalLanguage), code);
  end;

var
  i: Integer;
  thisList: TStringList;
begin
  thisList := TStringList.Create;
  try
    GetAvailableResources(thisList, '', FCompatible);

    if OriginalLanguage <> '' then
      Process(OriginalLanguage);

    for i := 0 to thisList.Count - 1 do
      Process(thisList[i]);
  finally
    thisList.Free;
  end;

  for i := 0 to LanguageCount - 1 do
  begin
    if ((LoadedResourceLocale <> '') and (Languages[i] = LoadedResourceLocale)) or
      ((LoadedResourceLocale = '') and (Languages[i] = OriginalLanguage)) then
    begin
      List.ItemIndex := i;
      Break;
    end;
  end;

  if (List.ItemIndex = -1) and (List.Items.Count > 0) then
    List.ItemIndex := 0;
end;

procedure TLaLanguageDialog.FormDestroy(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to List.Items.Count - 1 do
    TLanguageListItem(List.Items.Objects[i]).Free;
end;

procedure TLaLanguageDialog.ListDblClick(Sender: TObject);
begin
  ModalResult := mrOk;
end;

end.
