unit Main;

{$WARN UNSAFE_CAST OFF}

interface

uses
  Classes, Controls, StdCtrls, ComCtrls, Menus, Forms, LaCommon;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    ExitMenu: TMenuItem;
    HelpMenu: TMenuItem;
    AboutMenu: TMenuItem;
    StatusBar: TStatusBar;
    TypeLabel: TLabel;
    TypeCombo: TComboBox;
    MetricUnitLabel: TLabel;
    MetricCombo: TComboBox;
    MetricAmountLabel: TLabel;
    MetricEdit: TEdit;
    UsUnitLabel: TLabel;
    UsCombo: TComboBox;
    UsAmountLabel: TLabel;
    UsEdit: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure TypeComboChange(Sender: TObject);
    procedure MetricEditChange(Sender: TObject);
    procedure UsEditChange(Sender: TObject);
    procedure ExitMenuClick(Sender: TObject);
    procedure AboutMenuClick(Sender: TObject);

  private
    FUpdating: Boolean;
    
    function GetQuantity: TQuantity;

    function GetMetricLength: TLaMetricLength;
    function GetUsLength: TLaUsLength;

    function GetMetricTemperature: TLaMetricTemperature;
    function GetUsTemperature: TLaUsTemperature;

    function GetMetricValue: Double;
    procedure SetMetricValue(value: Double);

    function GetUsValue: Double;
    procedure SetUsValue(value: Double);

    procedure ShowHint(Sender: TObject);

  public
    property Quantity: TQuantity read GetQuantity;
    property MetricLength: TLaMetricLength read GetMetricLength;
    property UsLength: TLaUsLength read GetUsLength;
    property MetricTemperature: TLaMetricTemperature read GetMetricTemperature;
    property UsTemperature: TLaUsTemperature read GetUsTemperature;
    property MetricValue: Double read GetMetricValue write SetMetricValue;
    property UsValue: Double read GetUsValue write SetUsValue;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  SysUtils,
  Dialogs;

function TMainForm.GetQuantity: TQuantity;
begin
  Result := TQuantity(TypeCombo.Items.Objects[TypeCombo.ItemIndex]);
end;

function TMainForm.GetMetricLength: TLaMetricLength;
begin
  Result := TLaMetricLength(MetricCombo.Items.Objects[MetricCombo.ItemIndex]);
end;

function TMainForm.GetUsLength: TLaUsLength;
begin
  Result := TLaUsLength(UsCombo.Items.Objects[UsCombo.ItemIndex]);
end;

function TMainForm.GetMetricTemperature: TLaMetricTemperature;
begin
  Result := TLaMetricTemperature(MetricCombo.Items.Objects[MetricCombo.ItemIndex]);
end;

function TMainForm.GetUsTemperature: TLaUsTemperature;
begin
  Result := TLaUsTemperature(UsCombo.Items.Objects[UsCombo.ItemIndex]);
end;

function TMainForm.GetMetricValue: Double;
begin
  try
    Result := StrToFloat(MetricEdit.Text);
  except
    Result := 0;
  end;
end;

procedure TMainForm.SetMetricValue(value: Double);
begin
  FUpdating := True;
  try
    MetricEdit.Text := FloatToStrF(value, ffNumber, 10, 2);
  finally
    FUpdating := False;
  end;
end;

function TMainForm.GetUsValue: Double;
begin
  try
    Result := StrToFloat(UsEdit.Text);
  except
    Result := 0;
  end;
end;

procedure TMainForm.SetUsValue(value: Double);
begin
  FUpdating := True;
  try
    UsEdit.Text := FloatToStrF(value, ffNumber, 10, 2);
  finally
    FUpdating := False;
  end;
end;

procedure TMainForm.ShowHint(Sender: TObject);
begin
  StatusBar.SimpleText := GetLongHint(Application.Hint);
end;

procedure TMainForm.FormCreate(Sender: TObject);
const
  QUANTITIES: array[TQuantity] of String =
  (
    'Length',
    'Temperature'
  );
var
  q: TQuantity;
begin
  Application.OnHint := ShowHint;

  for q := Low(q) to High(q) do
    TypeCombo.AddItem(QUANTITIES[q], TObject(q));

  TypeCombo.ItemIndex := 0;
  TypeComboChange(Self);
end;

procedure TMainForm.TypeComboChange(Sender: TObject);

  procedure AddMetric(const str: String; value: Integer);
  begin
    MetricCombo.AddItem(str, TObject(value));
  end;

  procedure AddUs(const str: String; value: Integer);
  begin
    UsCombo.AddItem(str, TObject(value));
  end;

const
  METRIC_LENGHTS: array[TLaMetricLength] of String =
  (
    'Kilometers',
    'Meters',
    'Centimeters',
    'Millimeters'
  );

  US_LENGHTS: array[TLaUsLength] of String =
  (
    'Miles',
    'Yards',
    'Feet',
    'Inches'
  );

  METRIC_TEMPERATURES: array[TLaMetricTemperature] of String =
  (
    'Celsius',
    'Kelvin'
  );

  US_TEMPERATURES: array[TLaUsTemperature] of String =
  (
    'Fahrenheit'
  );
var
  ml: TLaMetricLength;
  ul: TLaUsLength;
  mt: TLaMetricTemperature;
  ut: TLaUsTemperature;
begin
  MetricCombo.Items.Clear;
  UsCombo.Items.Clear;

  case Quantity of
    qaLength:
    begin
      for ml := Low(ml) to High(ml) do
        AddMetric(METRIC_LENGHTS[ml], Ord(ml));

      for ul := Low(ul) to High(ul) do
        AddUs(US_LENGHTS[ul], Ord(ul));
    end;

    qaTemperature:
    begin
      for mt := Low(mt) to High(mt) do
        AddMetric(METRIC_TEMPERATURES[mt], Ord(mt));

      for ut := Low(ut) to High(ut) do
        AddUs(US_TEMPERATURES[ut], Ord(ut));
    end;
  end;

  MetricCombo.ItemIndex := 0;
  UsCombo.ItemIndex := 0;
end;

procedure TMainForm.MetricEditChange(Sender: TObject);
begin
  if FUpdating or (MetricEdit.Text = '') then
    Exit;

  case Quantity of
    qaLength:
      UsValue := MetricLengthToUsLength(MetricValue, MetricLength, UsLength);

    qaTemperature:
      UsValue := MetricTemperatureToUsTemperature(MetricValue, MetricTemperature, UsTemperature);
  end;
end;

procedure TMainForm.UsEditChange(Sender: TObject);
begin
  if FUpdating or (UsEdit.Text = '') then
    Exit;

  case Quantity of
    qaLength:
      MetricValue := UsLengthToMetricLength(UsValue, UsLength, MetricLength);

    qaTemperature:
      MetricValue := UsTemperatureToMetricTemperature(UsValue, UsTemperature, MetricTemperature);
  end;
end;

procedure TMainForm.ExitMenuClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.AboutMenuClick(Sender: TObject);
begin
  ShowMessage('Converter is an application that converts metric units to US units and back.');
end;

end.
