{
  Copyrights 2005-2011 Sisulizer

  Unit that translated the common dialogs.
  This compiles with Delphi 5 and later

  Note!
  When running on Vista or later most common dialog do not translate.

  The reason is that Windows common dialog are not real VCL components but
  wraps around WIN32 controls. They can not be translated as VCL components-
  We recommend not to use LaCommonDialogs any more.

  LaCommonDialogs is provied for backward compatibility only.
}

{$I LaDef.inc}

unit LaCommonDialogs;

interface

uses
  Windows;

function LaLocalizeSaveDialog(wnd: HWnd; reserved: Integer): Bool; stdcall;
function LaLocalizeOpenDialog(wnd: HWnd; reserved: Integer): Bool; stdcall;
function LaLocalizeColorDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
function LaLocalizeFontDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
function LaLocalizeReplaceDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
function LaLocalizeFindDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
function LaLocalizePrinterSetupDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
function LaLocalizePrintDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
{$IFDEF VCL7}
function LaLocalizePageSetupDialog(wnd: HWnd; reserved: Integer): Bool; stdcall;
{$ENDIF}

function LaLocalizeCommonDialogWindow(handle: HWND; msg: UINT; wParam: WPARAM; lParam: LPARAM): Longint; stdcall;

implementation

uses
  SysUtils,
  Messages,
  Classes,
  Controls,
  CommCtrl,
  Dialogs,
  ExtDlgs,
  LaCommon,
  LaResource;

resourcestring
  // Common strings for dialogs
  SName = 'Name';
  SSize = 'Size';
  SType = 'Type';
  SModified = 'Modified';
  SAttributes = 'Attributes';
  SCancel = 'Cancel';
  SHelp = '&Help';
  SOk = 'OK';
  SOpenCaption = 'Open';
  SSaveCaption = 'Save As';
  SUpOneLevel = 'Up One Level';
  SCreateNewFolder = 'Create New Folder';
  SList = 'List';
  SDetails = 'Details';
  SViewDesktop = 'View Desktop';
  SViewMenu = 'View Menu';
  SGoToLastFolderVisited = 'Go To Last Folder Visited';
  SMyRecentDocuments = 'My Recent Documents';
  SDesktop = 'Desktop';
  SMyDocuments = 'My Documents';
  SMyComputer = 'My Computer';
  SMyNetworkPlaces = 'My Network Places';

  // Strings for open dialog
  SOpenFilesOfType = 'Files of &type:';
  SOpenFileName = 'File &name:';
  SOpenLookIn = 'Look &in:';
  SOpenOpen = '&Open';
  SOpenAsReadOnly = 'Open as &read-only';

  // Strings for save dialog
  SSaveSave = '&Save';
  SSaveOpenAsReadOnly = 'Open as &read-only';
  SSaveAsType = 'Save as &type:';
  SSaveFileName = 'File &name:';
  SSaveIn = 'Save &in:';

  // Strings for color dialog
  SColorBasic = '&Basic colors:';
  SColorCustom = '&Custom colors:';
  SColorColor = 'Color';
  SColorAddCustom = '&Add to Custom Colors';
  SColorDefineCustom = '&Define Custom Colors >>';
  SColorRed = '&Red:';
  SColorGreen = '&Green:';
  SColorBlue = 'Bl&ue:';
  SColorHue = 'Hu&e:';
  SColorSat = '&Sat:';
  SColorLum = '&Lum:';
  SColorValue = 'Color';
  SColorSolid = '|S&olid';

  // Strings for font dialog
  SFontSample = 'Sample';
  SFontValue = '&Font:';
  SFontStyle = 'Font st&yle:';
  SFontSize = '&Size:';
  SFontFont = 'Font';
  SFontStrikeout = 'Stri&keout';
  SFontUnderline = '&Underline';
  SFontEffects = 'Effects';
  SFontColor = '&Color:';
  SFontScript = 'Sc&ript:';
  SFontApply = '&Apply';

  // Strings for find dialog
  SFindUp = '&Up';
  SFindDown = '&Down';
  SFindCaption = 'Find';
  SFindNext = '&Find Next';
  SFindDirection = 'Direction';
  SFindWhat = 'Fi&nd what:';
  SFindMatchWord = 'Match &whole word only';
  SFindMatchCase = 'Match &case';

  // Strings for replace dialog
  SReplaceFindWhat = 'Fi&nd what:';
  SReplaceTitle = 'Replace';
  SReplaceMatchWord = 'Match &whole word only';
  SReplaceMatchCase = 'Match &case';
  SReplaceFindNext = '&Find Next';
  SReplaceValue = '&Replace';
  SReplaceAll = 'Replace &All';
  SReplaceWith = 'Re&place with:';

  // Strings for print dialog
  SPrintStatus = 'Status:';
  SPrintComment = 'Comment:';
  SPrintPrint = 'Print';
  SPrintAll = '&All';
  SPrintSelection = '&Selection';
  SPrintPages = 'Pa&ges';
  SPrintProperties = '&Properties';
  SPrintToFile = 'Print to fi&le';
  SPrintCollate = 'C&ollate';
  SPrintRange = 'Print range';
  SPrintCopies = 'Copies';
  SPrintPrinter = 'Printer';
  SPrintName = '&Name:';
  SPrintFrom = '&from:';
  SPrintTo = '&to:';
  SPrintNumber = 'Number of &copies:';
  SPrintType = 'Type:';
  SPrintWhere = 'Where:';

  // Strings for printer dialog
  SPrinterNetwork = 'Net&work...';
  SPrinterPortrait = 'P&ortrait';
  SPrinterSetup = 'Print Setup';
  SPrinterProperties = '&Properties';
  SPrinterLandscape = 'L&andscape';
  SPrinterOrientation = 'Orientation';
  SPrinterSize = 'Si&ze:';
  SPrinterSource = '&Source:';
  SPrinterPaper = 'Paper';
  SPrinterValue = 'Printer';
  SPrinterName = '&Name:';
  SPrinterType = 'Type:';
  SPrinterComment = 'Comment:';
  SPrinterWhere = 'Where:';
  SPrinterStatus = 'Status:';

  // Strings for setup dialog
  SSetupPaper = 'Paper';
  SSetupSize = 'Si&ze:';
  SSetupCaption = 'Page Setup';
  SSetupPrinter = '&Printer...';
  SSetupOrientation = 'Orientation';
  SSetupSource = '&Source:';
  SSetupMarginsMm = 'Margins (millimeters)';
  SSetupMarginsIn = 'Margins (inches)';
  SSetupLeft = '&Left:';
  SSetupRight = '&Right:';
  SSetupTop = '&Top:';
  SSetupBottom = '&Bottom:';
  SSetupPortrait = 'P&ortrait';
  SSetupLandscape = 'L&andscape';

var
  FMethodOffset: Integer;
  FCounter: Integer;
  FClosing: Boolean;
  FTranslating: Boolean;
  FDialog: TCommonDialog;
  FOldDialogProc: Longint;
  FLocalizeableDialogs: TList;
  FVclDialogProcs: TList;
  FButtonProc: TFarProc;

type
  PLaToolTipTextA = PToolTipTextA;
  PLaToolTipTextW = PToolTipTextW;

{$IFNDEF VCL6}
  PPointer = ^Pointer;
{$ENDIF}

  TLaCommonDialog = class(TCommonDialog)
  protected
    procedure WndProc(var msg: TMessage); override;
  end;

procedure TLaCommonDialog.WndProc(var msg: TMessage);
var
  h: HWnd;
begin
  inherited;

  case msg.Msg of
    WM_CREATE:
      FClosing := False;

    WM_KILLFOCUS:
      if (FOldDialogProc = 0) and not FClosing then
      begin
        FDialog := Self;
        FOldDialogProc := SetWindowLong(Msg.WParam, GWL_WNDPROC, LongInt(@LaLocalizeCommonDialogWindow));
      end;

    WM_WINDOWPOSCHANGED:
      if FOldDialogProc = 0 then
      begin
        FClosing := False;
        FDialog := Self;
        h := GetActiveWindow;

        if (Owner = nil) or not (Owner is TWinControl) or (h <> TWinControl(Owner).Handle) then
        begin
          FOldDialogProc := SetWindowLong(h, GWL_WNDPROC, LongInt(@LaLocalizeCommonDialogWindow));
        end;
      end;

    WM_ENTERIDLE:
      if (FOldDialogProc = 0) and Self.InheritsFrom(TColorDialog) then
      begin
        FClosing := False;
        FDialog := Self;
        h := GetActiveWindow;

        if (Owner = nil) or not (Owner is TWinControl) or (h <> TWinControl(Owner).Handle) then
        begin
          FOldDialogProc := SetWindowLong(h, GWL_WNDPROC, LongInt(@LaLocalizeCommonDialogWindow));
          SendMessage(h, WM_ACTIVATE, 0, 0);
        end;
      end;
  end;
end;


procedure TranslateWindow(wnd: THandle; str: String; resize: Boolean);
var
  buffer: array[0..255] of Char;
  width, style: Integer;
  rect, calcRect: TRect;
  dc: HDC;
begin
  if str = '' then
  begin
    GetWindowText(wnd, buffer, Sizeof(buffer));
    str := buffer;
  end;

  SetWindowText(wnd, PChar(str));

  if resize then
  begin
    dc := GetWindowDC(wnd);
    try
      SelectObject(dc, GetStockObject(DEFAULT_GUI_FONT));
      width := DrawText(dc, PChar(str), -1, calcRect, DT_LEFT or DT_CALCRECT or DT_SINGLELINE);
    finally
      ReleaseDC(wnd, dc);
    end;

    if width <= 0 then
      Exit;

    width := calcRect.right - calcRect.left;
    GetClassName(wnd, buffer, SizeOf(buffer));
    StrLower(buffer);

    if buffer = 'button' then
    begin
      style := GetWindowLong(wnd, GWL_STYLE);

      if ((style and BS_CHECKBOX) <> 0) or ((style and BS_RADIOBUTTON) <> 0) then
        width := width + 20;
    end;

    GetWindowRect(wnd, rect);

    if (rect.right - rect.left) < width then
      SetWindowPos(
        wnd,
        0,
        0,
        0,
        width,
        rect.bottom - rect.top,
        SWP_NOMOVE or SWP_NOZORDER);
  end;
end;

procedure TranslateDialog(handle: HWnd);
begin
  FCounter := 0;
  FTranslating := True;
  try
    if (FDialog is TSaveDialog) or (FDialog is TSavePictureDialog) then
      LaLocalizeSaveDialog(handle, 0)
    else if (FDialog is TOpenDialog) or (FDialog is TOpenPictureDialog) then
      LaLocalizeOpenDialog(handle, 0)
    else if FDialog is TColorDialog then
      LaLocalizeColorDialog(handle, 0)
    else if FDialog is TFontDialog then
      LaLocalizeFontDialog(handle, 0)
    else if FDialog is TReplaceDialog then
      LaLocalizeReplaceDialog(handle, 0)
    else if FDialog is TFindDialog then
      LaLocalizeFindDialog(handle, 0)
    else if FDialog is TPrintDialog then
      LaLocalizePrintDialog(handle, 0)
    else if FDialog is TPrinterSetupDialog then
      LaLocalizePrinterSetupDialog(handle, 0)
{$IFDEF VCL7}
    else if FDialog is TPageSetupDialog then
      LaLocalizePageSetupDialog(handle, 0)
{$ENDIF}
  finally
    FTranslating := False;
  end;
end;

function WStrPCopy(dest: PWideChar; const source: UnicodeString): PWideChar;
var
  i, len: Integer;
begin
  Result := dest;
  len := Length(source);

  for i := 1 to len do
  begin
    dest^ := source[i];
    Inc(dest);
  end;

  dest^ := Chr(0);
end;

function LaLocalizeCommonDialogWindow(
  handle: HWND;
  msg: UINT;
  wParam: WPARAM;
  lParam: LPARAM): LongInt;
var
  tttW: PLaToolTipTextW;
  tttA: PLaToolTipTextA;
begin
  Result := 1;

  if (FOldDialogProc <> 0) and (msg <> WM_USER + $12C) then
    Result := CallWindowProc(Pointer(FOldDialogProc), handle, msg, wParam, lParam);

  case msg of
    WM_NOTIFY:
    begin
      if PNMHDR(LParam).code = TTN_NEEDTEXT then
      begin
        tttA := PLaToolTipTextA(LParam);

        case tttA^.hdr.idFrom of
          40961: StrPCopy(tttA^.szText, UnicodeToAnsi(SUpOneLevel));
          40962: StrPCopy(tttA^.szText, UnicodeToAnsi(SCreateNewFolder));
          40963: StrPCopy(tttA^.szText, UnicodeToAnsi(SList));
          40964: StrPCopy(tttA^.szText, UnicodeToAnsi(SDetails));
          40969: StrPCopy(tttA^.szText, UnicodeToAnsi(SViewDesktop));
          40970: StrPCopy(tttA^.szText, UnicodeToAnsi(SViewMenu));
          40971: StrPCopy(tttA^.szText, UnicodeToAnsi(SGoToLastFolderVisited));
        end;

        Result := 0;
        Exit;
      end
      else if PNMHDR(LParam).code = TTN_NEEDTEXTW then
      begin
        tttW := PLaToolTipTextW(LParam);

        case tttW^.hdr.idFrom of
          40961: WStrPCopy(tttW^.szText, SUpOneLevel);
          40962: WStrPCopy(tttW^.szText, SCreateNewFolder);
          40963: WStrPCopy(tttW^.szText, SList);
          40964: WStrPCopy(tttW^.szText, SDetails);
          40969: WStrPCopy(tttW^.szText, SViewDesktop);
          40970: WStrPCopy(tttW^.szText, SViewMenu);
          40971: WStrPCopy(tttW^.szText, SGoToLastFolderVisited);
        end;

        Result := 0;
        Exit;
      end;
    end;

    WM_CTLCOLORDLG, WM_ACTIVATE:
      if not FTranslating then
        TranslateDialog(handle);

    WM_COMMAND:
      if (LongRec(wParam).Lo = 1025) or (LongRec(wParam).Lo = 1024) then
        TranslateDialog(handle);

    WM_DESTROY, WM_CLOSE:
      if FOldDialogProc <> 0 then
      begin
        FClosing := True;
        SetWindowLong(handle, GWL_WNDPROC, FOldDialogProc);
        FOldDialogProc := 0;
      end;
  end;
end;

procedure TranslateToolBar(wnd: HWnd);
var
  str: String;
  count, i, id: Integer;
  button: TTBButton;
  buttonInfo: TTBButtonInfo;
begin
  buttonInfo.cbSize := Sizeof(buttonInfo);
  buttonInfo.dwMask := TBIF_TEXT;

  count := SendMessage(wnd, TB_BUTTONCOUNT, 0, 0);

  for i := 0 to count - 1 do
  begin
    if SendMessage(wnd, TB_GETBUTTON, i, LPARAM(@button)) <> 0 then
    begin
      id := button.idCommand;

      case id of
        41060: str := SMyRecentDocuments;
        41061: str := SDesktop;
        41062: str := SMyDocuments;
        41063: str := SMyComputer;
        41064: str := SMyNetworkPlaces;
      else
        str := '';
      end;

      if str <> '' then
      begin
        buttonInfo.pszText := PChar(str);
        SendMessage(wnd, TB_SETBUTTONINFO, id, LPARAM(@buttonInfo));
      end;
    end;
  end;
end;


procedure TranslateHeader(wnd: HWnd);
var
  str: String;
  count, i: Integer;
  headerItem: THDItem;
  buffer: array[0..255] of Char;
begin
  count := Header_GetItemCount(wnd);

  for i := 0 to count - 1 do
  begin
    headerItem.mask := HDI_TEXT;
    headerItem.pszText := buffer;
    headerItem.cchTextMax := SizeOf(buffer);

    Header_GetItem(wnd, i, headerItem);

    case i of
      0: str := SName;
      1: str := SSize;
      2: str := SType;
      3: str := SModified;
      4: str := SAttributes;
    else
      str := headerItem.pszText;
    end;

    if str <> buffer then
    begin
      headerItem.pszText := PChar(str);
      Header_SetItem(wnd, i, headerItem);
    end;
  end;
end;

function LaLocalizeOpenDialog(wnd: HWnd; reserved: Integer): Bool; stdcall;
var
  buffer: array[0..255] of Char;
begin
  Result := True;

  if TOpenDialog(FDialog).Title = '' then
    TranslateWindow(wnd, SOpenCaption, False);

  GetClassName(wnd, buffer, SizeOf(buffer));

  if buffer = TOOLBARCLASSNAME then
    TranslateToolBar(wnd)
  else if buffer = WC_HEADER then
    TranslateHeader(wnd)
  else if (buffer = 'Static') or (buffer = 'Button') then
  begin
    case GetWindowLong(wnd, GWL_ID) of
      1: TranslateWindow(wnd, SOpenOpen, True);
      2: TranslateWindow(wnd, SCancel, True);

      1038: TranslateWindow(wnd, SHelp, True);
      1040: TranslateWindow(wnd, SOpenAsReadOnly, True);
      1089: TranslateWindow(wnd, SOpenFilesOfType, True);
      1090: TranslateWindow(wnd, SOpenFileName, True);
      1091: TranslateWindow(wnd, SOpenLookIn, True);
    end;
  end;

  EnumChildWindows(wnd, @LaLocalizeOpenDialog, 0);
end;

function LaButtonProc(wnd: HWnd; msg: UINT; wParam: WPARAM; lParam: LPARAM): Integer; stdcall;
begin
  if msg = WM_SETTEXT then
    Result := Integer(True)
  else
    Result := CallWindowProc(FButtonProc, wnd, msg, wParam, lParam);
end;

function LaLocalizeSaveDialog(wnd: HWnd; reserved: Integer): Bool; stdcall;
var
  buffer: array[0..255] of Char;
begin
  Result := True;

  if TOpenDialog(FDialog).Title = '' then
    TranslateWindow(wnd, SSaveCaption, False);

  GetClassName(wnd, buffer, SizeOf(buffer));

  if buffer = TOOLBARCLASSNAME then
    TranslateToolBar(wnd)
  else if buffer = WC_HEADER then
    TranslateHeader(wnd)
  else if (buffer = 'Static') or (buffer = 'Button') then
  begin
    case GetWindowLong(wnd, GWL_ID) of
      1:
      begin
        TranslateWindow(wnd, SSaveSave, True);

        if not Assigned(FButtonProc) then
        begin
          FButtonProc := TFarProc(GetWindowLong(wnd, GWL_WNDPROC));
          SetWindowLong(wnd, GWL_WNDPROC, Integer(@LaButtonProc));
        end;
      end;

      2: TranslateWindow(wnd, SCancel, True);

      1038: TranslateWindow(wnd, SHelp, True);
      1040: TranslateWindow(wnd, SSaveOpenAsReadOnly, True);
      1089: TranslateWindow(wnd, SSaveAsType, True);
      1090: TranslateWindow(wnd, SSaveFileName, True);
      1091: TranslateWindow(wnd, SSaveIn, True);
    end;
  end;

  EnumChildWindows(wnd, @LaLocalizeSaveDialog, 0);
end;


function LaLocalizeColorDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
begin
  Result := True;

  case GetWindowLong(wnd, GWL_ID) of
    0: TranslateWindow(wnd, SColorColor, False);
    1: TranslateWindow(wnd, SOk, True);
    2: TranslateWindow(wnd, SCancel, True);

    712: TranslateWindow(wnd, SColorAddCustom, True);
    719: TranslateWindow(wnd, SColorDefineCustom, True);
    723: TranslateWindow(wnd, SColorHue, True);
    724: TranslateWindow(wnd, SColorSat, True);
    725: TranslateWindow(wnd, SColorLum, True);
    726: TranslateWindow(wnd, SColorRed, True);
    727: TranslateWindow(wnd, SColorGreen, True);
    728: TranslateWindow(wnd, SColorBlue, True);
    730: TranslateWindow(wnd, SColorValue, True);
    731: TranslateWindow(wnd, SColorSolid, True);

    1038: TranslateWindow(wnd, SHelp, True);

    65535:
      begin
        case FCounter of
          0: TranslateWindow(wnd, SColorBasic, True);
          1: TranslateWindow(wnd, SColorCustom, True);
        end;
        Inc(FCounter);
      end;
  end;

  EnumChildWindows(wnd, @LaLocalizeColorDialog, 0);
end;


function LaLocalizeFontDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
begin
  Result := True;

  case GetWindowLong(wnd, GWL_ID) of
    0: TranslateWindow(wnd, SFontFont, False);
    1: TranslateWindow(wnd, SOk, True);
    2: TranslateWindow(wnd, SCancel, True);

    1026: TranslateWindow(wnd, SFontApply, True);
    1038: TranslateWindow(wnd, SHelp, True);
    1040: TranslateWindow(wnd, SFontStrikeout, True);
    1041: TranslateWindow(wnd, SFontUnderline, True);
    1072: TranslateWindow(wnd, SFontEffects, True);
    1073: TranslateWindow(wnd, SFontSample, True);
    1088: TranslateWindow(wnd, SFontValue, True);
    1089: TranslateWindow(wnd, SFontStyle, True);
    1090: TranslateWindow(wnd, SFontSize, True);
    1091: TranslateWindow(wnd, SFontColor, True);
    1094: TranslateWindow(wnd, SFontScript, True);
  end;

  EnumChildWindows(wnd, @LaLocalizeFontDialog, 0);
end;


function LaLocalizePrintDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
begin
  Result := True;

  case GetWindowLong(wnd, GWL_ID) of
    0: TranslateWindow(wnd, SPrintPrint, False);
    1: TranslateWindow(wnd, SOk, True);
    2: TranslateWindow(wnd, SCancel, True);

    1025: TranslateWindow(wnd, SPrintProperties, True);
    1040: TranslateWindow(wnd, SPrintToFile, True);
    1041: TranslateWindow(wnd, SPrintCollate, True);
    1056: TranslateWindow(wnd, SPrintAll, True);
    1057: TranslateWindow(wnd, SPrintSelection, True);
    1058: TranslateWindow(wnd, SPrintPages, True);
    1072: TranslateWindow(wnd, SPrintRange, True);
    1073: TranslateWindow(wnd, SPrintCopies, True);
    1075: TranslateWindow(wnd, SPrintPrinter, True);
    1089: TranslateWindow(wnd, SPrintFrom, True);
    1090: TranslateWindow(wnd, SPrintTo, True);
    1092: TranslateWindow(wnd, SPrintNumber, True);
    1093: TranslateWindow(wnd, SPrintName, True);
    1094: TranslateWindow(wnd, SPrintType, True);
    1095: TranslateWindow(wnd, SPrintStatus, True);
    1096: TranslateWindow(wnd, SPrintComment, True);
    1097: TranslateWindow(wnd, SPrintWhere, True);
  end;

  EnumChildWindows(wnd, @LaLocalizePrintDialog, 0);
end;

function LaLocalizePrinterSetupDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
begin
  Result := True;

  case GetWindowLong(wnd, GWL_ID) of
    0: TranslateWindow(wnd, SPrinterSetup, False);
    1: TranslateWindow(wnd, SOk, True);
    2: TranslateWindow(wnd, SCancel, True);

    1025: TranslateWindow(wnd, SPrinterProperties, True);
    1037: TranslateWindow(wnd, SPrinterNetwork, True);
    1056: TranslateWindow(wnd, SPrinterPortrait, True);
    1057: TranslateWindow(wnd, SPrinterLandscape, True);
    1072: TranslateWindow(wnd, SPrinterOrientation, True);
    1073: TranslateWindow(wnd, SPrinterPaper, True);
    1075: TranslateWindow(wnd, SPrinterValue, True);
    1089: TranslateWindow(wnd, SPrinterSize, True);
    1090: TranslateWindow(wnd, SPrinterSource, True);
    1093: TranslateWindow(wnd, SPrinterName, True);
    1094: TranslateWindow(wnd, SPrinterType, True);
    1095: TranslateWindow(wnd, SPrinterStatus, True);
    1096: TranslateWindow(wnd, SPrinterComment, True);
    1097: TranslateWindow(wnd, SPrinterWhere, True);
  end;

  EnumChildWindows(wnd, @LaLocalizePrinterSetupDialog, 0);
end;

{$IFDEF VCL7}
function LaLocalizePageSetupDialog(wnd: HWnd; reserved: Integer): Bool; stdcall;
var
  units: TPageMeasureUnits;
begin
  Result := True;

  case GetWindowLong(wnd, GWL_ID) of
    0: TranslateWindow(wnd, SSetupCaption, False);
    1: TranslateWindow(wnd, SOK, False);
    2: TranslateWindow(wnd, SCancel, False);
    1026: TranslateWindow(wnd, SSetupPrinter, False);
    1073: TranslateWindow(wnd, SSetupPaper, False);
    1089: TranslateWindow(wnd, SSetupSize, True);
    1090: TranslateWindow(wnd, SSetupSource, True);
    1072: TranslateWindow(wnd, SSetupOrientation, False);
    1056: TranslateWindow(wnd, SSetupPortrait, True);
    1057: TranslateWindow(wnd, SSetupLandscape, True);

    1075:
    begin
      units := TPageSetupDialog(FDialog).Units;

      if units = pmDefault then
      begin
        if MeasurementSystem = msMetric then
          units := pmMillimeters
        else
          units := pmInches;
      end;

      if units = pmMillimeters then
        TranslateWindow(wnd, SSetupMarginsMm, False)
      else
        TranslateWindow(wnd, SSetupMarginsIn, False);
    end;

    1102: TranslateWindow(wnd, SSetupLeft, True);
    1103: TranslateWindow(wnd, SSetupRight, True);
    1104: TranslateWindow(wnd, SSetupTop, True);
    1105: TranslateWindow(wnd, SSetupBottom, True);
  end;

  EnumChildWindows(wnd, @LaLocalizePageSetupDialog, 0);
end;
{$ENDIF}

function LaLocalizeFindDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
begin
  Result := True;

  case GetWindowLong(wnd, GWL_ID) of
    0: TranslateWindow(wnd, SFindCaption, False);
    1: TranslateWindow(wnd, SFindNext, True);
    2: TranslateWindow(wnd, SCancel, True);

    1038: TranslateWindow(wnd, SHelp, True);
    1040: TranslateWindow(wnd, SFindMatchWord, True);
    1041: TranslateWindow(wnd, SFindMatchCase, True);
    1056: TranslateWindow(wnd, SFindUp, True);
    1057: TranslateWindow(wnd, SFindDown, True);
    1072: TranslateWindow(wnd, SFindDirection, True);

    65535: TranslateWindow(wnd, SFindWhat, True);
  end;

  EnumChildWindows(wnd, @LaLocalizeFindDialog, 0);
end;

function LaLocalizeReplaceDialog(wnd: HWnd; reserved: LPARAM): Bool; stdcall;
begin
  Result := True;

  case GetWindowLong(wnd, GWL_ID) of
    0: TranslateWindow(wnd, SReplaceTitle, False);
    1: TranslateWindow(wnd, SReplaceFindNext, True);
    2: TranslateWindow(wnd, SCancel, True);

    1038: TranslateWindow(wnd, SHelp, True);
    1024: TranslateWindow(wnd, SReplaceValue, True);
    1025: TranslateWindow(wnd, SReplaceAll, True);
    1040: TranslateWindow(wnd, SReplaceMatchWord, True);
    1041: TranslateWindow(wnd, SReplaceMatchCase, True);

    65535:
      begin
        case FCounter of
          0: TranslateWindow(wnd, SReplaceFindWhat, True);
          1: TranslateWindow(wnd, SReplaceWith, True);
        end;
        Inc(FCounter);
      end;
  end;

  EnumChildWindows(wnd, @LaLocalizeReplaceDialog, 0);
end;

procedure ChangeProcedure(memoryAddress: PPointer; procedureAddress: Pointer);
var
  oldProtect: Longint;
begin
  VirtualProtect(memoryAddress, 4, PAGE_READWRITE, @oldProtect);
  try
    memoryAddress^ := procedureAddress;
  finally
    VirtualProtect(memoryAddress, 4, oldProtect, @oldProtect);
  end;
end;

{
type
  TDialogData = record
    MethodAddress: PPointer;
    Data: TProcedureData;
  end;
  PDialogData = ^TDialogData;
}

function RegisterDialogClass(dialog: TClass): Boolean;
var
  methodAddress: PPointer;
  //dialogData: PDialogData;
begin
  if (dialog = nil) or not dialog.InheritsFrom(TCommonDialog) or
    (FLocalizeableDialogs.IndexOf(Pointer(dialog)) <> -1) then
  begin
    Result := False;
    Exit;
  end;

  if FMethodOffset = 0 then
    while (@TLaCommonDialog.WndProc <> PPointer(LongInt(TLaCommonDialog) + FMethodOffset)^) do
      Inc(FMethodOffset);

  FLocalizeableDialogs.Add(Pointer(dialog));

{
  dialogData := AllocMem(Sizeof(TDialogData));
  dialogData.MethodAddress := PPointer(Longint(dialog) + Longint(FMethodOffset));
  RemapProcedure(dialogData.MethodAddress^, @TLaCommonDialog.WndProc, dialogData.Data);
  FVclDialogProcs.Add(dialogData);
}
  methodAddress := PPointer(Longint(dialog) + Longint(FMethodOffset));
  ChangeProcedure(methodAddress, @TLaCommonDialog.WndProc);
  FVclDialogProcs.Add(methodAddress^);

  Result := True;
end;

function UnregisterDialogClass(dialog: TClass): Boolean;
var
  index: LongInt;
  //dialogData: PDialogData;
begin
  index := FLocalizeableDialogs.IndexOf(Pointer(dialog));

  if index = -1 then
  begin
    Result := False;
    Exit;
  end;

{
  dialogData := FVclDialogProcs[index];
  RestoreProcedure(dialogData.MethodAddress, dialogData.Data);
}
  ChangeProcedure(PPointer(LongInt(dialog) + FMethodOffset), FVclDialogProcs.Items[index]);

  FLocalizeableDialogs.Delete(index);
  FVclDialogProcs.Delete(index);

  Result := True;
end;

initialization
  FLocalizeableDialogs := TList.Create;
  FVclDialogProcs := TList.Create;

  FMethodOffset := 0;
  FOldDialogProc := 0;
  FClosing := False;

  RegisterDialogClass(TOpenDialog);
  RegisterDialogClass(TSaveDialog);
  RegisterDialogClass(TOpenPictureDialog);
  RegisterDialogClass(TSavePictureDialog);
  RegisterDialogClass(TColorDialog);
  RegisterDialogClass(TSaveDialog);
  RegisterDialogClass(TFontDialog);
  RegisterDialogClass(TPrintDialog);
  RegisterDialogClass(TPrinterSetupDialog);
  RegisterDialogClass(TFindDialog);
  RegisterDialogClass(TReplaceDialog);
{$IFDEF VCL7}
  RegisterDialogClass(TPageSetupDialog);
{$ENDIF}

finalization
  while FLocalizeableDialogs.Count > 0 do
    UnregisterDialogClass(TClass(FLocalizeableDialogs.Items[0]));

  FLocalizeableDialogs.Free;
  FVclDialogProcs.Free;
end.
